#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim:fileencoding=utf-8
"""
Created by Devyn Collier Johnson
<DevynCJohnson@Gmail.com>
LGPLv3 License
-- --
Create Desktop Entry Files
http://standards.freedesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html
http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html
"""


import os
import stat
import sys

from gi.repository import Gtk


__license__ = """GNU Lesser General Public License v3
Copyright (c) Devyn Collier Johnson, All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.
"""

__author__ = 'Devyn Collier Johnson'

__copyright__ = 'LGPLv3'

__version__ = '2015.11.07'


_GUI = """<?xml version="1.0" encoding="UTF-8"?>
<!-- Generated with glade 3.18.3

Copyright (C) LGPLv3

This file is part of Desktop-Entry-Maker.

Desktop-Entry-Maker is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Desktop-Entry-Maker is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Desktop-Entry-Maker.  If not, see <http://www.gnu.org/licenses/>.

Author: Devyn Collier Johnson

-->
<interface>
  <requires lib="gtk+" version="3.12"/>
  <!-- interface-license-type lgplv3 -->
  <!-- interface-name Desktop-Entry-Maker -->
  <!-- interface-description Create Desktop Entry Files -->
  <!-- interface-copyright LGPLv3 -->
  <!-- interface-authors Devyn Collier Johnson -->
  <object class="GtkApplicationWindow" id="desktop_entry_window">
    <property name="visible">True</property>
    <property name="can_focus">False</property>
    <property name="title" translatable="yes">Desktop-Entry Maker</property>
    <property name="default_width">600</property>
    <property name="default_height">600</property>
    <property name="icon_name">applications-development</property>
    <property name="show_menubar">False</property>
    <signal name="damage-event" handler="_winexit" swapped="no"/>
    <signal name="delete-event" handler="_winexit" swapped="no"/>
    <signal name="destroy" handler="_winexit" swapped="no"/>
    <signal name="destroy-event" handler="_winexit" swapped="no"/>
    <child>
      <object class="GtkBox" id="box1">
        <property name="visible">True</property>
        <property name="can_focus">False</property>
        <property name="margin_left">5</property>
        <property name="margin_right">5</property>
        <property name="margin_top">5</property>
        <property name="margin_bottom">5</property>
        <property name="orientation">vertical</property>
        <property name="spacing">5</property>
        <child>
          <object class="GtkGrid" id="entry_grid">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="column_spacing">5</property>
            <property name="row_homogeneous">True</property>
            <child>
              <object class="GtkLabel" id="label_name">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Application's Name (Required)</property>
                <property name="label" translatable="yes">Name</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_name">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Application's Name (Required)</property>
                <property name="hexpand">True</property>
                <property name="primary_icon_tooltip_text" translatable="yes">The program's name (will be seen in menus)</property>
                <property name="placeholder_text" translatable="yes">*Required*</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_execute">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Command or path to executable (Required)
Arguments
%f - Single file
%F - File list
%u - Single URL
%U - URL List</property>
                <property name="label" translatable="yes">Execute</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">2</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_execute">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Command or path to executable (Required)
Arguments
%f - Single file
%F - File list
%u - Single URL
%U - URL List</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">*Required*</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">2</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_genericname">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Used to indicate the application's general purpose
Example: LibreOffice = Office Suite</property>
                <property name="label" translatable="yes">Generic Name</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">1</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_genericname">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Used to indicate the application's general purpose
Example: LibreOffice = Office Suite</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">Office Suite</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">1</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_version">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Application's version number (optional)</property>
                <property name="label" translatable="yes">Version</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">3</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_version">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Application's version number (optional)</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">1.0</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">3</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_comment">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Tooltip comment (optional)</property>
                <property name="label" translatable="yes">Comment</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">4</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_comment">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Tooltip comment (optional)</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">Use a descriptive comment</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">4</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_category">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Application categories (separated with a semicolon)
Examples:
Development
Education
Game
GTK
Java
Office
Settings
System
Utility</property>
                <property name="label" translatable="yes">Categories</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">7</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_category">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Application categories (separated with a semicolon)
Examples:
Development
Education
Game
GTK
Java
Office
Settings
System
Utility</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">System;Development;</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">7</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_nodisplay">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Do not add application to "open with" and desktop/application menus</property>
                <property name="label" translatable="yes">NoDisplay</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">9</property>
              </packing>
            </child>
            <child>
              <object class="GtkCheckButton" id="check_nodisplay">
                <property name="label" translatable="yes">True</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">False</property>
                <property name="tooltip_text" translatable="yes">Do not add application to "open with" and desktop/application menus</property>
                <property name="halign">center</property>
                <property name="xalign">0</property>
                <property name="draw_indicator">True</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">9</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_terminal">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Run in a terminal?</property>
                <property name="label" translatable="yes">Terminal</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">10</property>
              </packing>
            </child>
            <child>
              <object class="GtkCheckButton" id="check_terminal">
                <property name="label" translatable="yes">True</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">False</property>
                <property name="tooltip_text" translatable="yes">Run in a terminal?</property>
                <property name="halign">center</property>
                <property name="xalign">0</property>
                <property name="draw_indicator">True</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">10</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_type">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Entry Type such as "Application", "Link", or "Directory" (Required)</property>
                <property name="label" translatable="yes">Type</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">8</property>
              </packing>
            </child>
            <child>
              <object class="GtkComboBoxText" id="combobox_type">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Entry Type such as "Application", "Link", or "Directory" (Required)</property>
                <property name="hexpand">True</property>
                <property name="active">0</property>
                <items>
                  <item translatable="yes">Application</item>
                  <item translatable="yes">Link</item>
                  <item translatable="yes">Directory</item>
                </items>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">8</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_icon">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Icon path or icon name (optional)
$HOME/.icons
$XDG_DATA_DIRS/icons
/usr/share/pixmaps</property>
                <property name="label" translatable="yes">Icon</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">6</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_icon">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Icon path or icon name (optional)
$HOME/.icons
$XDG_DATA_DIRS/icons
/usr/share/pixmaps</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">Name or Path (/usr/share/icons/)</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">6</property>
              </packing>
            </child>
            <child>
              <object class="GtkLabel" id="label_keywords">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="tooltip_text" translatable="yes">Keywords for searching (optional)
Separate words with a semicolon</property>
                <property name="label" translatable="yes">Keywords</property>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">5</property>
              </packing>
            </child>
            <child>
              <object class="GtkEntry" id="entry_keyword">
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="tooltip_text" translatable="yes">Keywords for searching (optional)
Separate words with a semicolon</property>
                <property name="hexpand">True</property>
                <property name="placeholder_text" translatable="yes">editor;cpu;programming;</property>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">5</property>
              </packing>
            </child>
          </object>
          <packing>
            <property name="expand">True</property>
            <property name="fill">True</property>
            <property name="position">0</property>
          </packing>
        </child>
        <child>
          <object class="GtkSeparator" id="separator1">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="fill">True</property>
            <property name="padding">5</property>
            <property name="position">1</property>
          </packing>
        </child>
        <child>
          <object class="GtkGrid" id="btn_grid">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="column_homogeneous">True</property>
            <child>
              <object class="GtkButton" id="btn_close">
                <property name="label">gtk-close</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">True</property>
                <property name="tooltip_text" translatable="yes">Close this application</property>
                <property name="use_stock">True</property>
                <property name="image_position">right</property>
                <property name="always_show_image">True</property>
                <signal name="clicked" handler="_winexit" object="desktop_entry_window" swapped="no"/>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
            <child>
              <object class="GtkButton" id="_save_as">
                <property name="label">gtk-save-as</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">True</property>
                <property name="tooltip_text" translatable="yes">Create the Desktop-Entry file in the selected location</property>
                <property name="use_stock">True</property>
                <property name="always_show_image">True</property>
                <signal name="clicked" handler="_save_as" object="desktop_entry_window" swapped="no"/>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="fill">True</property>
            <property name="pack_type">end</property>
            <property name="position">2</property>
          </packing>
        </child>
        <child>
          <object class="GtkLabel" id="label_about">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="label" translatable="yes">LGPLv3
Devyn Collier Johnson
&lt;DevynCJohnson@Gmail.com&gt;
DCJTech.info
Version 2015.11.07</property>
            <property name="justify">center</property>
            <property name="selectable">True</property>
            <property name="track_visited_links">False</property>
            <attributes>
              <attribute name="style" value="oblique"/>
            </attributes>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="fill">True</property>
            <property name="position">3</property>
          </packing>
        </child>
      </object>
    </child>
  </object>
</interface>
"""

_FILECHOOSER = """<?xml version="1.0" encoding="UTF-8"?>
<!-- Generated with glade 3.18.3

Copyright (C) LGPLv3

This file is part of File Chooser.

File Chooser is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

File Chooser is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with File Chooser.  If not, see <http://www.gnu.org/licenses/>.

Author: Devyn Collier Johnson

-->
<interface>
  <requires lib="gtk+" version="3.12"/>
  <!-- interface-license-type lgplv3 -->
  <!-- interface-name File Chooser -->
  <!-- interface-description Select a file -->
  <!-- interface-copyright LGPLv3 -->
  <!-- interface-authors Devyn Collier Johnson -->
  <object class="GtkApplicationWindow" id="ezwin">
    <property name="visible">True</property>
    <property name="can_focus">False</property>
    <property name="halign">baseline</property>
    <property name="valign">baseline</property>
    <property name="title" translatable="yes">Choose a File</property>
    <property name="window_position">center-on-parent</property>
    <property name="default_width">600</property>
    <property name="default_height">500</property>
    <property name="destroy_with_parent">True</property>
    <property name="icon_name">system-file-manager</property>
    <property name="type_hint">dialog</property>
    <property name="show_menubar">False</property>
    <signal name="delete-event" handler="_winexit" object="ezwin" swapped="no"/>
    <signal name="destroy-event" handler="_winexit" object="ezwin" swapped="no"/>
    <child>
      <object class="GtkBox" id="box1">
        <property name="visible">True</property>
        <property name="can_focus">False</property>
        <property name="halign">baseline</property>
        <property name="valign">baseline</property>
        <property name="margin_left">5</property>
        <property name="margin_right">5</property>
        <property name="margin_top">5</property>
        <property name="margin_bottom">5</property>
        <property name="orientation">vertical</property>
        <child>
          <object class="GtkFileChooserWidget" id="filechooser">
            <property name="width_request">500</property>
            <property name="visible">True</property>
            <property name="can_focus">True</property>
            <property name="create_folders">False</property>
            <property name="preview_widget_active">False</property>
            <property name="use_preview_label">False</property>
          </object>
          <packing>
            <property name="expand">True</property>
            <property name="fill">True</property>
            <property name="position">0</property>
          </packing>
        </child>
        <child>
          <object class="GtkSeparator" id="separator1">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="margin_top">5</property>
            <property name="margin_bottom">5</property>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="fill">True</property>
            <property name="position">1</property>
          </packing>
        </child>
        <child>
          <object class="GtkCheckButton" id="chkbtn2">
            <property name="label" translatable="yes">Select current location</property>
            <property name="visible">True</property>
            <property name="can_focus">True</property>
            <property name="receives_default">False</property>
            <property name="tooltip_text" translatable="yes">Marked - The current location/folder is returned

Empty - The selected folder is returned</property>
            <property name="halign">center</property>
            <property name="valign">center</property>
            <property name="xalign">0</property>
            <property name="draw_indicator">True</property>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="fill">True</property>
            <property name="position">2</property>
          </packing>
        </child>
        <child>
          <object class="GtkGrid" id="grid1">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="halign">baseline</property>
            <property name="valign">baseline</property>
            <property name="margin_left">5</property>
            <property name="margin_right">5</property>
            <property name="margin_top">5</property>
            <property name="column_homogeneous">True</property>
            <child>
              <object class="GtkButton" id="btn1">
                <property name="label">gtk-ok</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">False</property>
                <property name="tooltip_text" translatable="yes">Submit selection to the program that opened this dialog.</property>
                <property name="halign">start</property>
                <property name="valign">center</property>
                <property name="margin_top">5</property>
                <property name="margin_bottom">5</property>
                <property name="use_stock">True</property>
                <property name="always_show_image">True</property>
                <signal name="clicked" handler="_select" object="filechooser" swapped="no"/>
              </object>
              <packing>
                <property name="left_attach">0</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
            <child>
              <object class="GtkCheckButton" id="chkbtn1">
                <property name="label" translatable="yes">View hidden files</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">False</property>
                <property name="tooltip_text" translatable="yes">Marked - Show hidden files
Empty - Hidden files not seen</property>
                <property name="halign">center</property>
                <property name="valign">center</property>
                <property name="xalign">0</property>
                <property name="draw_indicator">True</property>
                <signal name="toggled" handler="_hidden" object="filechooser" swapped="no"/>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
            <child>
              <object class="GtkButton" id="btn2">
                <property name="label">gtk-cancel</property>
                <property name="visible">True</property>
                <property name="can_focus">True</property>
                <property name="receives_default">False</property>
                <property name="tooltip_text" translatable="yes">Cancel</property>
                <property name="halign">end</property>
                <property name="valign">center</property>
                <property name="margin_top">5</property>
                <property name="margin_bottom">5</property>
                <property name="use_stock">True</property>
                <property name="image_position">right</property>
                <property name="always_show_image">True</property>
                <signal name="clicked" handler="_cancel" object="ezwin" swapped="no"/>
              </object>
              <packing>
                <property name="left_attach">2</property>
                <property name="top_attach">0</property>
              </packing>
            </child>
          </object>
          <packing>
            <property name="expand">False</property>
            <property name="fill">True</property>
            <property name="position">4</property>
          </packing>
        </child>
      </object>
    </child>
  </object>
</interface>
"""


def write2file(_filename: str, _write) -> None:
    """Send data to new file or overwrite file"""
    with open(_filename, 'w') as _file:
        _file.write(str(_write))
    return


def ezfilech() -> list or str:
    """File Chooser Dialog"""
    _init_path = ''
    _viewhidden = False
    _local = True
    filewin = Gtk.Builder()
    filewin.add_from_string(buffer=_FILECHOOSER)
    _filew = filewin.get_object('filechooser')
    _chkbtn_hidden = filewin.get_object('chkbtn1')
    _chkbtn_dir = filewin.get_object('chkbtn2')
    _out = []
    _chkbtn_dir.set_visible(False)
    _filew.set_create_folders(True)
    _filew.set_local_only(_local)
    _filew.set_show_hidden(_viewhidden)
    _chkbtn_hidden.set_active(_viewhidden)
    if _init_path:
        try:
            _filew.set_current_folder(_init_path)
        except (OSError, NameError, TypeError, ValueError):
            pass
    _filew.set_action(Gtk.FileChooserAction.SAVE)
    _filew.set_do_overwrite_confirmation(True)

    # Handler Functions
    def _winexit(*_x) -> None:
        """Close application"""
        nonlocal _out
        _out = 'Exit'
        filewin.get_object('ezwin').destroy()
        Gtk.main_quit()
        return

    def _cancel(*_x) -> None:
        """Cancel and close"""
        nonlocal _out
        _out = 'Cancel'
        filewin.get_object('ezwin').destroy()
        Gtk.main_quit()
        return

    def _select(*_x) -> None:
        """Select files"""
        nonlocal _out
        _out = ' '.join(_filew.get_filenames())
        filewin.get_object('ezwin').destroy()
        Gtk.main_quit()
        return

    def _hidden(_widget, *_x) -> None:
        """Show hidden files"""
        _widget.set_show_hidden(not _widget.get_show_hidden())
        return

    filewin.connect_signals({
        '_winexit': _winexit,
        '_cancel': _cancel,
        '_select': _select,
        '_hidden': _hidden,
    })
    Gtk.main()
    return _out


class MainWin():
    """Main Window"""
    def __init__(self):
        """Main window"""
        self.ui = Gtk.Builder()
        self.ui.add_from_string(buffer=_GUI)
        # Match signal to function (handler)
        dic = {
            '_winexit': Gtk.main_quit,
            '_save_as': self._save_as,
        }
        self.ui.connect_signals(dic)

    # Buttons
    def _save_as(self, *_x) -> None:
        """Save XDG *.desktop file"""
        # Get objects
        entry_name = self.ui.get_object('entry_name')
        entry_genericname = self.ui.get_object('entry_genericname')
        entry_execute = self.ui.get_object('entry_execute')
        entry_icon = self.ui.get_object('entry_icon')
        entry_version = self.ui.get_object('entry_version')
        combobox_type = self.ui.get_object('combobox_type')
        check_terminal = self.ui.get_object('check_terminal')
        check_nodisplay = self.ui.get_object('check_nodisplay')
        entry_comment = self.ui.get_object('entry_comment')
        entry_keyword = self.ui.get_object('entry_keyword')
        entry_category = self.ui.get_object('entry_category')
        # Get Data
        _entry_name = entry_name.get_text()
        _entry_genericname = entry_genericname.get_text()
        _entry_execute = entry_execute.get_text()
        _entry_icon = entry_icon.get_text()
        _entry_version = entry_version.get_text()
        _combobox_type = str(combobox_type.get_active_text())
        _check_terminal = str(check_terminal.get_active()).casefold()
        _check_nodisplay = str(check_nodisplay.get_active()).casefold()
        _entry_comment = entry_comment.get_text()
        _entry_keyword = entry_keyword.get_text()
        _entry_category = entry_category.get_text()
        if len(_entry_name) == 0 or len(_entry_execute) == 0:
            return
        elif not _entry_name[0].isalnum():
            return
        if len(_entry_version) == 0:
            _entry_version = '1.0'
        if len(_entry_comment) == 0:
            _entry_comment = _entry_name
        if 'none' in _check_terminal.casefold():
            _check_terminal = 'false'
        if 'none' in _check_nodisplay.casefold():
            _check_nodisplay = 'false'
        if 'link' in _combobox_type.casefold():
            _entry_type = str(
                '\nType=Link' +
                '\nURL=' + _entry_execute
            )
        elif 'directory' in _combobox_type.casefold():
            _entry_type = '\nType=Directory'
        else:
            _entry_type = str(
                '\nType=Application' +
                '\nExec=' + _entry_execute
            )
        # Create Data
        _data = str(
            '#!/usr/bin/env xdg-open\n' +
            '[Desktop Entry]' +
            '\nName=' + _entry_name +
            '\nGenericName=' + _entry_genericname +
            _entry_type +
            '\nNoDisplay=' + _check_nodisplay +
            '\nVersion=' + _entry_version +
            '\nCategories=' + _entry_category +
            '\nTerminal=' + _check_terminal +
            '\nIcon=' + _entry_icon +
            '\nComment=' + _entry_comment +
            '\nKeywords=' + _entry_keyword +
            '\n'
        )
        # Select Location
        _path = ezfilech()
        if _path.casefold() == 'cancel' or _path.casefold() == 'none':
            return
        if _path == '':
            return
        # Write Data
        try:
            if _path.endswith('.desktop'):
                write2file(_path, _data)
            else:
                _path = _path + '.desktop'
                write2file(_path, _data)
        except:
            return
        # Set Permissions
        _st = os.stat(_path)
        os.chmod(_path, _st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
        # Close/End
        Gtk.main_quit()
        return


def main() -> None:
    """Open window and run program"""
    window = MainWin()
    Gtk.main()
    return


if __name__ == '__main__':
    main()
    sys.exit(0)
